package com.cool.bang.modules.mysql.service;

import com.cool.bang.common.util.DateUtil;
import com.cool.bang.modules.mysql.domain.*;
import com.cool.bang.modules.mysql.domain.flowPo.ApprovalFlow;
import com.cool.bang.modules.mysql.domain.flowPo.ApprovalNode;
import com.cool.bang.modules.mysql.domain.flowPo.ApprovalRecord;
import com.cool.bang.modules.mysql.mapper.*;
import com.cool.bang.modules.oracle.domain.Areg;
import com.cool.bang.modules.oracle.domain.AregItem;
import com.cool.bang.modules.oracle.domain.Bxmxy;
import com.cool.bang.modules.oracle.domain.Trade;
import com.cool.bang.modules.oracle.domain.po.*;
import com.cool.bang.modules.oracle.mapper.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Service
public class MySQLSecondService {

    @Autowired
    private MySqlMapper mySqlMapper;
    @Autowired
    private OracleMapper oracleMapper;
    @Autowired
    private OrgMapper qyxxMapper;
    @Autowired
    private AccountMapper accountMapper;
    ;
    @Autowired
    private ProjectAccountMapper projectAccountMapper;
    @Autowired
    private BuildingAccountMapper buildingAccountMapper;


    /**
     * 退款
     */
    @Transactional
    public void refundTo() {
        System.out.println("===============fc_refund 导入数据开始=================");
        List<TkInfo> htTkList = oracleMapper.selectHtTk();
        List<Refund> refunds = new ArrayList<>();
        for (TkInfo item : htTkList) {
            Integer qybh = StringUtils.isEmpty(item.getQybh()) ? null : Integer.valueOf(item.getQybh());
            Org org = qyxxMapper.selectOrgByMisid(qybh);
            if (org == null) { //测试用
                org = new Org();
                org.setId(3);
                org.setOrgName("山东酷邦软件有限公司");
            }
            BuildingAccount buildingAccount = null;
            if (!StringUtils.isEmpty(item.getLzbh())) {
                buildingAccount = buildingAccountMapper.getInfoByLZBH(item.getLzbh());
            }
            Long projectAccountId = (null != buildingAccount && null != buildingAccount.getProjectAccountId()) ? buildingAccount.getProjectAccountId() : null;
            ProjectAccount projectAccount = null;
            if (null != projectAccountId) {
                projectAccount = projectAccountMapper.getInfoById(projectAccountId);
            }
            Account account = null;
            if (!StringUtils.isEmpty(item.getAccountNo())) {
                account = accountMapper.selectByAccountNO(item.getAccountNo());
            }
            ContractInfo contract = null;
            if (!StringUtils.isEmpty(item.getHtBh())) {
                contract = mySqlMapper.getInfoByNumber(item.getHtBh());
            }
            Refund refund = new Refund(item, org, projectAccount, account, contract, buildingAccount);
            //定金退款的
            if (null == item.getMoney()) {
                refund.setAppropriationStatus("0");
            } else {
                refund.setAppropriationStatus("1");
            }
            refunds.add(refund);
        }
        mySqlMapper.insertRefund(refunds);
        System.out.println("===============fc_refund数据匹配结束=================");
    }


    @Transactional
    public void tradeDDZTo() {
        System.out.println("===============fc_trade 导入数据开始=================");
        List<TradeInfo> tradeInfoList = oracleMapper.selectDDZ();
        List<FcTrade> fcTrades = new ArrayList<>();

        for (TradeInfo item : tradeInfoList) {
            BuildingAccount buildingAccount = null;
            if (!StringUtils.isEmpty(item.getLzbh())) {
                buildingAccount = buildingAccountMapper.getInfoByLZBH(item.getLzbh());
            }
            Long projectAccountId = (null != buildingAccount && null != buildingAccount.getProjectAccountId()) ? buildingAccount.getProjectAccountId() : null;
            ProjectAccount projectAccount = null;
            if (null != projectAccountId) {
                projectAccount = projectAccountMapper.getInfoById(projectAccountId);
            }
            Account account = null;
            if (!StringUtils.isEmpty(item.getAccountNo())) {
                account = accountMapper.selectByAccountNO(item.getAccountNo());
            }

            FcTrade fcTrade = new FcTrade(item, projectAccount, account, buildingAccount);

            fcTrades.add(fcTrade);
        }
        mySqlMapper.insertFcTrade(fcTrades);
        System.out.println("===============fc_trade数据匹配结束=================");
    }

    @Transactional
    public void tradeTgTo() {
        System.out.println("===============fc_trade 导入数据开始=================");
        List<TradeInfo> tradeInfoList = oracleMapper.selectTG();

        List<FcTrade> fcTrades = new ArrayList<>();

        for (TradeInfo item : tradeInfoList) {
            BuildingAccount buildingAccount = null;
            if (!StringUtils.isEmpty(item.getLzbh())) {
                buildingAccount = buildingAccountMapper.getInfoByLZBH(item.getLzbh());
            }
            Long projectAccountId = (null != buildingAccount && null != buildingAccount.getProjectAccountId()) ? buildingAccount.getProjectAccountId() : null;
            ProjectAccount projectAccount = null;
            if (null != projectAccountId) {
                projectAccount = projectAccountMapper.getInfoById(projectAccountId);
            }
            Account account = null;

            if (!StringUtils.isEmpty(item.getAccountNo())) {
                account = accountMapper.selectByAccountNO(item.getAccountNo());
            }

            FcTrade fcTrade = new FcTrade(item, projectAccount, account, buildingAccount);

            fcTrades.add(fcTrade);
        }
        mySqlMapper.insertFcTrade(fcTrades);
        System.out.println("===============fc_trade数据匹配结束=================");
    }

    /**
     * 对账
     */
    public void accountChecking() {
        List<Map<String, Object>> tgXm = mySqlMapper.selectTgXm();
        List<String> list = new ArrayList<>();
        for (Map<String, Object> map : tgXm) {
            List<Map<String, Object>> xmAccount = mySqlMapper.selectXmAccountByProjectId((Long) map.get("id"));
            for (Map<String, Object> account : xmAccount) {
                List<Map<String, Object>> buildingInfo = mySqlMapper.selectBuildingBy((Long) map.get("id"), (Long) account.get("id"));
                for (Map<String, Object> building : buildingInfo) {
                    //监管转托管
                    BigDecimal zgToTg = oracleMapper.sumByZgToTg((String) (building.get("building_number")));
                    zgToTg = null == zgToTg ? BigDecimal.ZERO : zgToTg;
                    //交存
                    BigDecimal jc = oracleMapper.sumByJc((String) building.get("building_number"));
                    jc = null == jc ? BigDecimal.ZERO : jc;
                    //拨付
                    BigDecimal bf = oracleMapper.sumByBf((String) building.get("building_number"));
                    bf = null == bf ? BigDecimal.ZERO : bf;

                    BigDecimal accountBalance = mySqlMapper.getAccountBalanceByNumber((String) building.get("building_number"));
                    accountBalance = accountBalance == null ? BigDecimal.ZERO : accountBalance;
                    BigDecimal hj = (zgToTg.add(jc)).subtract(bf);
                    StringBuilder str = new StringBuilder();
                    String buildingName = String.valueOf(building.get("building_name"));
                    BigDecimal cz = BigDecimal.ZERO;
                    if (!(hj.compareTo(accountBalance) == 0)) {
                        cz = hj.subtract(accountBalance);
                    }
                    str.append(building.get("project_name").toString())
                            .append(";" + building.get("project_number").toString())
                            .append(";" + buildingName)
                            .append(";" + building.get("building_number").toString())
                            .append(";" + zgToTg)
                            .append(";" + jc)
                            .append(";" + bf)
                            .append(";" + hj)
                            .append(";" + accountBalance)
                            .append(";" + (hj.compareTo(accountBalance) == 0 ? "是" : "否"))
                            .append(";" + (hj.compareTo(accountBalance) == 0 ? "" : cz));
                    System.out.println(str.toString());
                    list.add(str.toString());
                }
            }
        }
        for (String str : list) {
            System.out.println(str);
        }
    }

    /**
     * 银行账户对账
     */
    public void bankAccountChecing() {
        List<Account> list = mySqlMapper.selectTgAccountInfo();
        List<Account> wdzJc = oracleMapper.getWdzJc();
        List<Account> wdzBf = oracleMapper.getWdzBf();
        List<String> strs = new ArrayList<>();
        //处理特殊的对账
        //1、寿光农商银行  缺失的 定金626475.6 (银行流水号 = 041002202216625184199811) 手续费0.44（银行流水号= 041002202216655621675571） (总的楼栋户的合计 比 账户余额少 这两笔 需要加上这两笔 )
        //2、中国建设银行股份有限公司寿光支行 bankCode：02   手续费 -200(银行流水号= 021002202216625378315471) （总的楼栋户的合计 比对完账的账户余额 多200 总的楼栋户合计需要加上这个-200）
        //3、中国邮政储蓄银行股份有限公司寿光市支行 利息186893.99 (银行流水号=161002202316730870521161) （总的楼栋户的合计 比账户余额少这一笔  总的楼栋户合计需要加上这个 186893.99）
        List<String> serialNumbers = new ArrayList<>();
        serialNumbers.add("041002202216625184199811");
        serialNumbers.add("041002202216655621675571");
        serialNumbers.add("021002202216625378315471");
        serialNumbers.add("161002202316730870521161");
        List<Account> tsList = mySqlMapper.selectTsDzByserial(serialNumbers);
        for (Account account : list) {

            //未对账的交存
            BigDecimal jc = BigDecimal.ZERO;
            if (wdzJc != null && !wdzJc.isEmpty()) {
                Account jcAccount = wdzJc.stream().filter(t -> !StringUtils.isEmpty(t.getBankCode()) && t.getBankCode().equals(account.getBankCode())).findFirst().orElse(null);
                jc = null != jcAccount ? jcAccount.getBalance() : BigDecimal.ZERO;
            }
            //未对账的拨付
            BigDecimal bf = BigDecimal.ZERO;
            if (wdzBf != null && !wdzBf.isEmpty()) {
                Account bfAccount = wdzBf.stream().filter(t -> !StringUtils.isEmpty(t.getBankCode()) && t.getBankCode().equals(account.getBankCode())).findFirst().orElse(null);
                bf = null != bfAccount ? bfAccount.getBalance() : BigDecimal.ZERO;
            }
            //当前托管户 的其他类型的 对账金额
            BigDecimal otherTypeAmount = BigDecimal.ZERO;
            Account otherTypeAccount = tsList.stream().filter(t -> !StringUtils.isEmpty(t.getBankCode()) && t.getBankCode().equals(account.getBankCode())).findFirst().orElse(null);
            if (otherTypeAccount != null) {
                otherTypeAmount = otherTypeAccount.getBalance();
                otherTypeAmount = null != otherTypeAmount ? otherTypeAccount.getBalance() : BigDecimal.ZERO;
            }

            StringBuilder str = new StringBuilder();
            str.append(account.getBankName())
                    .append(";" + account.getBankCode())
                    .append(";" + jc)
                    .append(";" + bf)
                    .append(";" + otherTypeAmount)
                    .append(";" + account.getBalance())
                    .append(";" + account.getAccountBalance())
                    .append(";" + ((account.getBalance().subtract(jc)).add(bf).add(otherTypeAmount)).subtract(account.getAccountBalance()));
            strs.add(str.toString());
        }

        for (String str : strs) {
            System.out.println(str);
        }
    }

    public void contractFundCheckTrade() {
        List<FundInfo> tgFund = mySqlMapper.getTgFund();
        List<String> strs = new ArrayList<>();
        for (FundInfo item : tgFund) {
            //查询 交易
            BigDecimal amount = mySqlMapper.getAmountByBusinessNumber(item.getBusinessNumber());
            StringBuilder str = new StringBuilder();
            str.append(item.getContractNumber() + ";")
                    .append(item.getContractId() + ";")
                    .append(item.getBusinessNumber() + ";")
                    .append(1 == item.getPaymentStatus() ? "未足额" : (2 == item.getPaymentStatus() ? "已交存" : (0 == item.getPaymentStatus() ? "未交存" : "")))
                    .append(";" + item.getPayableAmount() + ";")
                    .append(amount + ";")
                    .append(item.getPayableAmount().compareTo(amount) == 0 ? "是" : "否");
            strs.add(str.toString());
        }
        System.out.flush();
        for (String str : strs) {
            System.out.println(str);
        }
    }

}
